/*
** conv.c
**
** Part of RPN calculator
** Copyright (c) 2005, 2006, 2008, 2010 NuukiaWorld
**
**  RPN calculator is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

/* includes nokix headers */
#include <nokix/all.h>
#include <math.h>

#include "main.h"
#include "stack.h"
#include "calc.h"

/*******************************************************/

/*******************************************************/

static const double temp_offset[] = { 273.15, 459.67, 0, 0 };

/*******************************************************/

void conv_init (void) {

	struct menu_item item = {
		NULL, NULL, NULL, 0, 0, OMSG_CONV_UNIT, action_ok_back
	};
	conv_data_t *data;

	// Delete old items (if any)
	while (menu_get_item (1, 1)) menu_rem_item(1, 1);

	data = conv_data = (conv_data_t*)message.argv[0];
	conv_unit = -1;						// No "Convert from" unit selected yet

	// Add new items
	do {
		item.title = (nokstr_t*)data->name;
		menu_add_item (1, 0, &item);
	} while (item.arg++, (++data)->name != 0);
}

/* Sets the flag value of all conv menu items
- in "Convert from" menu flags must be 0x00
- in "Convert to" menu flags must be 0x10
0x04 could be used in both menus but in that case shortcut access will
pause to display the selected item also in the last menu (which is not
useful as the selected item can be seen in the message displayed)
*/
void conv_flags( unsigned int flags ) {

	unsigned int c;
   struct menu_item *item;

   for( c = 1; ( item = menu_get_item( 1, c ) ); c++ ) item->flags = flags;
}

void conv_calc(void) {

   double d;

	if( !stack_arg_check(1) ) return;
   d = stack->stack[0];
   
	if( conv_data == conv_temperature ) d += temp_offset[conv_unit];             // from temp
	if( &conv_data[conv_unit] == &conv_dbm ) d = pow( 10.0, d / 10.0 );          // from dBm
	d *= conv_data[conv_unit].factor / conv_data[msg_argv[0]].factor;
	if( &conv_data[msg_argv[0]] == &conv_dbm ) d = log(d) / M_LN10 * 10.0;       // to dBm
	if( conv_data == conv_temperature ) d -= temp_offset[msg_argv[0]];           // to temp

   if( !check_value(d) ) return;
   stack_dup(0);
   stack->stack[0] = d;
   
	// Inform the user about the conversion
	display_msg( DMI_DONE, (const nokstr_t*)w_sprintf( &str_converted,
		conv_data[conv_unit].name, conv_data[message.argv[0]].name ) );
}

