/*
** main.h
**
** Part of RPN calculator
** Copyright (c) 2005, 2007, 2008, 2010 NuukiaWorld
**
**  RPN calculator is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

/*******************************************************/

/* Define 32-bit custom types that are efficient for ARM7 registers.
   Use inside functions and in function prototypes when there is no
   need for truncation to a specific number of bits!
*/
typedef  unsigned int char_t;  // Use instead of char
typedef  unsigned int bool_t;  // Use instead of bool

/*******************************************************/

typedef struct {
   unsigned short depth;         // Current stack depth
   unsigned short alloc_depth;   // Allocated stack depth
   double         stack[];       // Stack contents
} stack_t;

typedef const struct {
	const char *name[2];   // name[0] = name, name[1] = symbol+unit
	double     value;
} const_data_t;

typedef const struct {
	const char *name;
	double     factor;
} conv_data_t;

struct globals {
// Beginning of global_buffer is used also in exchange rate dialog!
   char	buf[16];
   unsigned char  state;
   unsigned char  edit_ptr;
   unsigned char  sign_ptr;
   signed char	   conv_unit;
   unsigned short stack_ptr;
   unsigned short	stack_bot;
   struct font    *font1;
   stack_t        *last_stack;
   conv_data_t    *conv_data;
   signed char    radix;
   unsigned char  wait_hex;
   unsigned char  const_mode;
};

/*******************************************************/

extern const nokstr_t str_empty;
extern const nokstr_t str_last_stack;
extern const nokstr_t str_memory;
extern const nokstr_t str_arguments;
extern const nokstr_t str_range;
extern const nokstr_t str_invalid_date;
extern const nokstr_t str_negative;
extern const nokstr_t str_select_dest_unit;
extern const nokstr_t str_converted;
extern const nokstr_t str_syntax;

extern const struct menu menu_options;
extern const struct menu menu_stack;

extern const struct bitmap bmp_cursor;
extern const conv_data_t conv_dbm;
extern const conv_data_t conv_temperature[];

extern rexx_uint action_options_back;
extern rexx_uint action_options_drop;
extern rexx_uint action_options_clear;
extern rexx_uint action_ok_clear;

extern rexx_uint eeprid_exchange_rate;
extern rexx_uint id_exchange_rate;

extern rexx_uint confid_angle_unit;
extern rexx_uint confid_radix;

/*******************************************************/

#define  buf	      (((struct globals*)global_buffer)->buf)
#define  state       (((struct globals*)global_buffer)->state)
#define  edit_ptr    (((struct globals*)global_buffer)->edit_ptr)
#define  sign_ptr    (((struct globals*)global_buffer)->sign_ptr)
#define  conv_unit	(((struct globals*)global_buffer)->conv_unit)
#define  stack_ptr   (((struct globals*)global_buffer)->stack_ptr)
#define  stack_bot	(((struct globals*)global_buffer)->stack_bot)
#define  font1       (((struct globals*)global_buffer)->font1)
#define  last_stack  (((struct globals*)global_buffer)->last_stack)
#define  conv_data	(((struct globals*)global_buffer)->conv_data)
#define  radix       (((struct globals*)global_buffer)->radix)
#define  wait_hex    (((struct globals*)global_buffer)->wait_hex)
#define  const_mode  (((struct globals*)global_buffer)->const_mode)

#define  MAX_EDIT       15			// Max length of buf[]

#define  STATE_MENU		3
#define	STATE_SHORTCUT	2			// Shortcut session open
#define  STATE_EDIT		4			// Command line mode
#define	STATE_STACK		8			// Interactive stack mode
#define	STATE_CONST		16 		// Constant list open

#define  RADIX_16       0        // Used for radix
#define  RADIX_8        1
#define  RADIX_2        2

/* own messages should start with MSG_OWN, then MSG_OWN+1 and so on */
#define  OMSG_STACK        MSG_OWN     // Stack operation menu item
#define  OMSG_CALC         (MSG_OWN+1) // Calculation operation selected
#define  OMSG_UNDO         (MSG_OWN+2) // Undo selected

#define  OMSG_DROP			(MSG_OWN+5) // Drop in normal mode
#define  OMSG_EXCHANGE		(MSG_OWN+6) // Disables numeric keys
#define  OMSG_CONV_MENU		(MSG_OWN+7) // Conversion menu selected
#define  OMSG_CONV_UNIT		(MSG_OWN+8) // Conversion unit selected
#define  OMSG_CONST_MENU	(MSG_OWN+9) // Constants menu selected
#define  OMSG_CONST		   (MSG_OWN+10)// Constant selected

/*******************************************************/

/*******************************************************/


