/*
** math.c
**
** Part of RPN calculator
** Copyright (c) 2006, 2007 NuukiaWorld
**
**  RPN calculator is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include "fdlibm.h"
#include "float.h"

/*******************************************************/

/* Missing math functions are implemented here
*/
double gamma1p( double x ) {

   int sign;

   x = __ieee754_exp( __ieee754_lgamma_r( x + 1, &sign ) );
   return ( sign < 0 ? -x : x );
}

/* Check the out-of-range condition of built-in log(x)
*/
double log( double x ) {

   return ( x > 0 ? __ieee754_log(x) : DBL_MAX + DBL_MAX );
}

/* Calculate accurate log(x+1) value using built-in log(x)
*/
double log1p( double x ) {

   double u;

   u = 1.0 + x;
	if( u == 1.0 ) return x;
	return log(u) * ( x / ( u - 1.0 ) );
}

/* Use expm1(x) to calculate __ieee754_exp(x)
*/
double __ieee754_exp( double x ) {

   return expm1(x) + 1;
}


